SecretKey

Struct SecretKey 

pub struct SecretKey<C: Curve> {
    inner: ScalarValue<C>,
}
Elliptic curve secret keys.

This type wraps a secret scalar value, helping to prevent accidental exposure and securely erasing the value from memory when dropped.

Generating secret keys

SecretKeys can be generated using the Generate trait.

Parsing PKCS#8 Keys

PKCS#8 is a commonly used format for encoding secret keys (especially ones generated by OpenSSL).

Keys in PKCS#8 format are either binary (ASN.1 BER/DER), or PEM encoded (ASCII) and begin with the following:

-----BEGIN PRIVATE KEY-----

To decode an elliptic curve private key from PKCS#8, enable the pkcs8 feature of this crate (or the pkcs8 feature of a specific RustCrypto elliptic curve crate) and use the DecodePrivateKey trait to parse it.

When the pem feature of this crate (or a specific RustCrypto elliptic curve crate) is enabled, a FromStr impl is also available.

Fields

inner: ScalarValue<C>

Scalar value

Implementations

Source

impl<C> SecretKey<C>
where C: Curve,

pub const MIN_SIZE: usize = 24

Minimum allowed size of an elliptic curve secret key in bytes.

This provides the equivalent of 96-bits of symmetric security.

pub fn from_scalar(scalar: impl Into<ScalarValue<C>>) -> CtOption<Self>

Create a new secret key from a scalar value.

Returns

This will return a none if the scalar is all-zero.

pub fn as_scalar_value(&self) -> &ScalarValue<C>

Borrow the inner secret ScalarValue value.

Security Warning

This value is key material. Please treat it with the care it deserves!

pub fn to_nonzero_scalar(&self) -> NonZeroScalar<C>
where C: CurveArithmetic,

Get the secret NonZeroScalar value for this key.

Security Warning

This value is key material. Please treat it with the care it deserves!

pub fn public_key(&self) -> PublicKey<C>
where C: CurveArithmetic,

Get the PublicKey which corresponds to this secret key

pub fn from_bytes(bytes: &FieldBytes<C>) -> Result<Self>

Deserialize secret key from an encoded secret scalar.

Errors

Returns Error if bytes overflows the scalar modulus.

pub fn from_slice(slice: &[u8]) -> Result<Self>

Deserialize secret key from an encoded secret scalar passed as a byte slice.

The slice is expected to be a minimum of 24-bytes (192-bytes) and at most C::FieldBytesSize bytes in length.

Byte slices shorter than the field size are handled by zero padding the input.

NOTE: this function is variable-time with respect to the input length. To avoid a timing sidechannel, always ensure that the input has been pre-padded to C::FieldBytesSize.

Errors
  • if slice is shorter than Self::MIN_SIZE.
  • if slice is not sized appropriately for the scalar modulus.
  • if the decoded scalar field element overflows the scalar modulus.
pub fn to_bytes(&self) -> FieldBytes<C>

Serialize raw secret scalar as a big endian integer.

pub fn diffie_hellman(&self, public_key: &PublicKey<C>) -> SharedSecret<C>
where C: CurveArithmetic,

Perform Elliptic Curve Diffie-Hellman with the given public key, returning a shared secret.

See the documentation in the ecdh module for more information.

pub fn from_der(der_bytes: &[u8]) -> DecodeResult<Self>

Decode SecretKey from DER-encoded private key.

Supported formats
  • PKCS#8: requires pkcs8 feature
  • SEC1: requires sec1 feature

If you are expecting one format over the other, it’s better to use a more specific method:

Errors
pub fn from_pem(pem: &str) -> DecodeResult<Self>

Decode SecretKey from PEM-encoded private key.

If you are expecting one format over the other, it’s better to use a more specific method:

Errors
pub fn from_sec1_der(der_bytes: &[u8]) -> DecodeResult<Self>

Deserialize secret key encoded in the SEC1 ASN.1 DER ECPrivateKey format.

Errors
  • if der_bytes does not encode a valid SEC1 private key
  • if the contained document does not encode a valid key for this curve
pub fn to_sec1_der(&self) -> Result<Zeroizing<Vec<u8>>>

Serialize secret key in the SEC1 ASN.1 DER ECPrivateKey format.

Errors

If an internal error occurs encoding this key.

pub fn from_sec1_pem(s: &str) -> DecodeResult<Self>

Parse SecretKey from PEM-encoded SEC1 ECPrivateKey format.

PEM-encoded SEC1 keys can be identified by the leading delimiter:

-----BEGIN EC PRIVATE KEY-----
Errors
  • if the document cannot be decoded as PEM
  • if the PEM document does not encode a valid private key for this curve
pub fn to_sec1_pem(&self, line_ending: LineEnding) -> Result<Zeroizing<String>>

Serialize private key as self-zeroizing PEM-encoded SEC1 ECPrivateKey with the given pem::LineEnding.

Pass Default::default() to use the OS’s native line endings.

Errors

If an internal error occurs encoding this key.

pub fn random<R: CryptoRng + ?Sized>(rng: &mut R) -> Self

👎Deprecated since 0.14.0: use the Generate trait instead

Deprecated: Generate a random SecretKey.